@@ -32,7 +32,8 @@ from hypothesis.errors import Flaky, Timeout, NoSuchExample, \
     UnsatisfiedAssumption, HypothesisDeprecationWarning
 from hypothesis.control import BuildContext
 from hypothesis._settings import settings as Settings
-from hypothesis._settings import Phase, Verbosity, HealthCheck
+from hypothesis._settings import Phase, Verbosity, HealthCheck, \
+    note_deprecation
 from hypothesis.executors import new_style_executor, \
     default_new_style_executor
 from hypothesis.reporting import report, verbose_report, current_verbosity
@@ -42,7 +43,7 @@ from hypothesis.internal.compat import str_to_bytes, get_type_hints, \
 from hypothesis.utils.conventions import infer
 from hypothesis.internal.escalation import \
     escalate_hypothesis_internal_error
-from hypothesis.internal.reflection import nicerepr, arg_string, \
+from hypothesis.internal.reflection import is_mock, nicerepr, arg_string, \
     impersonate, function_digest, fully_qualified_name, \
     define_function_signature, convert_positional_arguments, \
     get_pretty_function_description
@@ -255,8 +256,11 @@ def fail_health_check(settings, message, label):
     # Tell pytest to omit the body of this function from tracebacks
     # http://doc.pytest.org/en/latest/example/simple.html#writing-well-integrated-assertion-helpers
     __tracebackhide__ = True
+
     if label in settings.suppress_health_check:
         return
+    if not settings.perform_health_check:
+        return
     message += (
         '\nSee https://hypothesis.readthedocs.io/en/latest/health'
         'checks.html for more information about this. '
@@ -422,6 +426,12 @@ def process_arguments_to_given(
         selfy = kwargs.get(argspec.args[0])
     elif arguments:
         selfy = arguments[0]
+
+    # Ensure that we don't mistake mocks for self here.
+    # This can cause the mock to be used as the test runner.
+    if is_mock(selfy):
+        selfy = None
+
     test_runner = new_style_executor(selfy)
 
     arguments = tuple(arguments)
@@ -488,6 +498,9 @@ def new_given_argspec(original_argspec, generator_kwargs):
         args=new_args, kwonlyargs=new_kwonlyargs, annotations=annots)
 
 
+HUNG_TEST_TIME_LIMIT = 5 * 60
+
+
 class StateForActualGivenExecution(object):
 
     def __init__(self, test_runner, search_strategy, test, settings, random):
@@ -502,6 +515,15 @@ class StateForActualGivenExecution(object):
         self.random = random
 
     def evaluate_test_data(self, data):
+        if (
+            time.time() - self.start_time >= HUNG_TEST_TIME_LIMIT
+        ):
+            fail_health_check(self.settings, (
+                'Your test has been running for at least five minutes. This '
+                'is probably not what you intended, so by default Hypothesis '
+                'turns it into an error.'
+            ), HealthCheck.hung_test)
+
         try:
             result = self.test_runner(data, reify_and_execute(
                 self.search_strategy, self.test,
@@ -530,7 +552,7 @@ class StateForActualGivenExecution(object):
         # Tell pytest to omit the body of this function from tracebacks
         __tracebackhide__ = True
         database_key = str_to_bytes(fully_qualified_name(self.test))
-        start_time = time.time()
+        self.start_time = time.time()
         runner = ConjectureRunner(
             self.evaluate_test_data,
             settings=self.settings, random=self.random,
@@ -538,20 +560,26 @@ class StateForActualGivenExecution(object):
         )
         runner.run()
         note_engine_for_statistics(runner)
-        run_time = time.time() - start_time
-        timed_out = (
-            self.settings.timeout > 0 and
-            run_time >= self.settings.timeout
-        )
+        run_time = time.time() - self.start_time
+        timed_out = runner.exit_reason == ExitReason.timeout
         if runner.last_data is None:
             return
         if runner.last_data.status == Status.INTERESTING:
             self.falsifying_example = runner.last_data.buffer
-            if self.settings.database is not None:
-                self.settings.database.save(
-                    database_key, self.falsifying_example
-                )
         else:
+            if timed_out:
+                note_deprecation((
+                    'Your tests are hitting the settings timeout (%.2fs). '
+                    'This functionality will go away in a future release '
+                    'and you should not rely on it. Instead, try setting '
+                    'max_examples to be some value lower than %d (the number '
+                    'of examples your test successfully ran here). Or, if you '
+                    'would prefer your tests to run to completion, regardless '
+                    'of how long they take, you can set the timeout value to '
+                    'hypothesis.unlimited.'
+                ) % (
+                    self.settings.timeout, runner.valid_examples),
+                    self.settings)
             if runner.valid_examples < min(
                 self.settings.min_satisfying_examples,
                 self.settings.max_examples,
